// Canvas3d.java

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

class Canvas3d extends Canvas
{
    Model3d md;
    Image backBuffer;
    Graphics backGC;
    Dimension prefSize, minSize;
    boolean painted;

    Canvas3d(Model3d md)
    {
        if (md == null)
            return;
        this.md = md;
        prefSize = new Dimension(md.width, md.height);
        minSize  = new Dimension(md.width / 2, md.height / 2);
    }

    public Dimension minimumSize() 
    {
        return minSize;
    }

    public Dimension preferredSize()
    {
        return prefSize;
    }
    
    public void update(Graphics g)
    {
        paint(g);
    }

    public void paint(Graphics g) 
    {
        backGC.setColor(getBackground());
        backGC.fillRect(0, 0, size().width, size().height);
        if (md != null)
            md.paint(backGC);
        g.drawImage(backBuffer, 0, 0, this);
        setPainted();
    }
    
    synchronized void setPainted() 
    {
        painted = true;
        notifyAll();
    }

    synchronized void waitPainted() 
    {
        while (!painted)
            try 
            {   
                wait(); 
            } 
            catch (Exception e) 
            {
            }
        painted = false;
    }

    public void reshape(int  x, int  y, int  width, int  height)
    {
        super.reshape(x, y, width, height);
        backBuffer = createImage(width, height);
        backGC = backBuffer.getGraphics();
        if (md != null)
            md.resize(width, height);
    }

    void releaseModel()
    {
        md = null;
    }

}
