// Vector3d.java

class Vector3d 
{
    float x, y, z;

    Vector3d() 
    {
    }

    Vector3d(Vector3d v)
    {
        copy(v);
    }

    Vector3d(float x, float y, float z) 
    { 
        set(x, y, z);
    }

    void copy(Vector3d v)
    {
        x = v.x;
        y = v.y;
        z = v.z;
    }

    void set(float x, float y, float z)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    void add(Vector3d v)
    { 
        x += v.x; y += v.y; z += v.z;
    }

    void sub(Vector3d v)
    { 
        x -= v.x; y -= v.y; z -= v.z;
    }

    void cmul(Vector3d v)
    { 
        x *= v.x; y *= v.y; z *= v.z;
    }

    float dot(Vector3d v)
    { 
        return x * v.x + y * v.y + z * v.z;
    }

    void mul(Vector3d v)
    {
        float tx = x, ty = y, tz = z;
        x = ty * v.z - tz * v.y;
        y = tz * v.x - tx * v.z;
        z = tx * v.y - ty * v.x;
    }

    void mul(float f)
    { 
        x *= f; y *= f; z *= f;
    }

    void div(float f)
    { 
        x /= f; y /= f; z /= f;
    }

    boolean equals(Vector3d v)
    { 
        if (v == null)
            return false;
        return x == v.x && y == v.y && z == v.z;
    }

    float vabs()
    { 
        return (float) Math.sqrt(x * x + y * y + z * z);
    }

    void normalize()
    { 
        float t = vabs();
        x /= t; y /= t; z /= t;
    }

    float cos(Vector3d v)
    { 
        return dot(v) / (vabs() * v.vabs());
    }

    float sin(Vector3d v)
    { 
        Vector3d t = new Vector3d(this);
        t.mul(v);
        return t.vabs() / (vabs() * v.vabs());
    }

    public String toString()
    {
        return "[" + x + ", " + y + ", " + z + "]";
    }

}
